@@ -7,41 +7,35 @@
 #### Regression analysis with compositional data containing zero values
 #### Chilean Journal of Statistics, 6(2): 47-57
 ################################
-alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), K = 10, mat = NULL, nc = 1, graph = FALSE) {
+alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), nfolds = 10, folds = NULL, nc = 1,
+                         seed = FALSE, graph = FALSE) {
   ## y is the compositional data (dependent variable)
   ## x is the independent variables
   ## a is a range of values of alpha
-  ## K is the number of folds for the K-fold cross validation
+  ## nfolds is the number of folds for the K-fold cross validation
   ## nc is how many cores you want to use, default value is 2
   if ( min(y) == 0 )  a <- a[a>0]
   la <- length(a)
   n <- dim(y)[1]
+  ina <- 1:n
   x <- model.matrix(y ~., data.frame(x) )
+  if ( is.null(folds) )  folds <- Compositional::makefolds(ina, nfolds = nfolds,
+                                                           stratified = FALSE, seed = seed)
+  nfolds <- length(folds)
 
-  if ( is.null(mat) ) {
-    nu <- sample(1:n, min( n, round(n / K) * K ) )
-    ## It may be the case this new nu is not exactly the same
-    ## as the one specified by the user
-    ## to a matrix a warning message should appear
-    options(warn = -1)
-    mat <- matrix( nu, ncol = K ) # if the length of nu does not fit
-  } else  mat <- mat
-
-  K <- dim(mat)[2]
-
-  if (nc == 1) {
+  if (nc <= 1) {
     apa <- proc.time()
-    kula <- matrix(nrow = K, ncol = la)
+    kula <- matrix(nrow = nfolds, ncol = la)
     for (j in 1:la) {
       ytr <- alfa(y, a[j])$aff
-      for (i in 1:K) {
-        xu <- x[ mat[, i], -1 , drop = FALSE]
-        yu <- y[ mat[, i], ]
-        xa <- x[ -mat[, i], -1]
-        yb <- ytr[ -mat[, i], ]
+      for (i in 1:nfolds) {
+        xu <- x[ folds[[ i ]], -1 , drop = FALSE]
+        yu <- y[ folds[[ i ]], ]
+        xa <- x[ -folds[[ i ]], -1]
+        yb <- ytr[ -folds[[ i ]], ]
         mod <- alfa.reg(yu, xa, a[j], xnew = xu, yb = yb)
         yest <- mod$est
-        kula[i, j] <- 2 * sum(yu * log(yu / yest), na.rm = TRUE)
+        kula[i, j] <- 2 * mean(yu * log(yu / yest), na.rm = TRUE)
       }
     }
 
@@ -54,31 +48,34 @@ alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), K = 10, mat = NULL, nc
 
   } else {
     apa <- proc.time()
-    options(warn = -1)
+    oop <- options(warn = -1)
+	on.exit( options(oop) )
     val <- matrix(a, ncol = nc) ## if the length of a is not equal to the
     ## dimensions of the matrix val a warning message should appear
-    ## but with options(warn = -1) you will not see it
-    cl <- makePSOCKcluster(nc)
-    registerDoParallel(cl)
-    kula <- foreach(j = 1:nc, .combine = cbind, .packages = "Rfast", .export = c("alfa.reg",
+    ## but with oop <- options(warn = -1) you will not see it
+    cl <- parallel::makePSOCKcluster(nc)
+    doParallel::registerDoParallel(cl)
+    if ( is.null(folds) )  folds <- Compositional::makefolds(ina, nfolds = nfolds,
+                                                             stratified = FALSE, seed = seed)
+    kula <- foreach::foreach(j = 1:nc, .combine = cbind, .packages = "Rfast", .export = c("alfa.reg",
 	        "alfa", "helm", "comp.reg", "multivreg", "rowsums", "colmeans", "colVars") ) %dopar% {
        ba <- val[, j]
-       ww <- matrix(nrow = K, ncol = length(ba) )
+       ww <- matrix(nrow = nfolds, ncol = length(ba) )
        for ( l in 1:length(ba) ) {
           ytr <- alfa(y, ba[l])$aff
-          for (i in 1:K) {
-            xu <- x[ mat[, i], -1, drop = FALSE]
-            yu <- y[ mat[, i], ]
-            xa <- x[ -mat[, i], -1]
-            yb <- ytr[ -mat[, i], ]
+          for (i in 1:nfolds) {
+            xu <- x[ folds[[ i ]], -1 , drop = FALSE]
+            yu <- y[ folds[[ i ]], ]
+            xa <- x[ -folds[[ i ]], -1]
+            yb <- ytr[ -folds[[ i ]], ]
             mod <- alfa.reg(yu, xa, ba[l], xnew = xu, yb = yb)
             yest <- mod$est
-            ww[i, l] <- 2 * sum(yu * log(yu / yest), na.rm = TRUE)
+            ww[i, l] <- 2 * mean(yu * log(yu / yest), na.rm = TRUE)
           }
        }
        return(ww)
     }
-    stopCluster(cl)
+    parallel::stopCluster(cl)
 
     kula <- kula[, 1:la]
     kl <- Rfast::colmeans(kula)
@@ -92,7 +89,7 @@ alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), K = 10, mat = NULL, nc
   if ( graph ) {
     plot( a, kula[1, ], type = 'l', ylim = c( min(kula), max(kula) ), xlab = expression(alpha),
 	       ylab = 'Twice the Kullback Leibler divergence', cex.lab = 1.3 )
-    for (i in 2:K)  lines(a, kula[i, ])
+    for (i in 2:nfolds)  lines(a, kula[i, ])
     lines(a, kl, col = 2, lty = 2, lwd = 2)
   }
 
